require_relative '../../test_helper'

class AssertMoar::AssertionTest < Minitest::Test
  def setup
    super

    Minitest::Test.reset

    @tc = Minitest::Test.new('foo')
    @tc.send(:extend, AssertMoar::Assertions)
    @assertion_count = 1
  end

  def test_assert_valid
    @tc.assert_valid ActiveRecordDouble.new(valid: true)
  end

  def test_refute_valid
    @tc.refute_valid ActiveRecordDouble.new(valid: false)
  end

  def test_assert_validates_presence_of
    validator = ::ActiveModel::Validations::PresenceValidator
    double = ActiveRecordDouble.new(error_map: { property: validator })
    @tc.assert_validates_presence_of(double, :property)
  end

  def test_refute_validates_presence_of
    double = ActiveRecordDouble.new()
    @tc.refute_validates_presence_of(double, :property)
  end

  def test_assert_uniqueness_of
    validator = ::ActiveRecord::Validations::UniquenessValidator
    double = ActiveRecordDouble.new(error_map: { property: validator })
    @tc.assert_validates_uniqueness_of(double, :property)
  end

  def test_refute_uniqueness_of
    double = ActiveRecordDouble.new()
    @tc.refute_validates_uniqueness_of(double, :property)
  end

  def test_assert_validates_absence_of
    validator = ::ActiveModel::Validations::AbsenceValidator
    double = ActiveRecordDouble.new(error_map: {property: validator })
    @tc.assert_validates_absence_of(double, :property)
  end

  def test_refute_validates_absense_of
    double = ActiveRecordDouble.new()
    @tc.refute_validates_absence_of(double, :property)
  end

  def test_assert_validates_acceptance_of
    validator = ::ActiveModel::Validations::AcceptanceValidator
    double = ActiveRecordDouble.new(error_map: {property: validator})
    @tc.assert_validates_acceptance_of(double, :property)
  end

  def test_refute_validates_acceptance_of
    double = ActiveRecordDouble.new()
    @tc.refute_validates_acceptance_of(double, :property)
  end

  def test_assert_validates_with_callback
    validator = ::ActiveSupport::Callbacks::Callback.new(:callback)
    double = ActiveRecordDouble.new(error_map: { callback: validator })

    @tc.assert_validates_with_callback(double, :callback)
  end

  def test_refute_validates_with_callback
    validator = ::ActiveSupport::Callbacks::Callback.new(:callback)
    double = ActiveRecordDouble.new()

    @tc.refute_validates_with_callback(double, :callback)
  end

  def test_assert_memoizes_method_result
    double = ActiveRecordDouble.new()
    @tc.assert_memoizes_method_result(double, :memoized_method)
  end

  def test_refute_memoizes_method_result
    double = ActiveRecordDouble.new()
    @tc.refute_memoizes_method_result(double, :non_memoized_method)
  end

  def teardown
    assert_equal(@assertion_count, @tc.assertions,
                 "expected #{@assertion_count} assertions to be fired during the test, not #{@tc.assertions}") if @tc.passed?
  end
end
